package com.game.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

import java.rmi.ServerException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import com.game.business.domain.AppGame;
import com.game.business.domain.AppGameCommission;
import com.game.business.domain.AppUser;
import com.game.business.dto.AppAgentEditDto;
import com.game.business.dto.AppAgentTeamDto;
import com.game.business.service.IAppGameCommissionService;
import com.game.business.service.IAppGameService;
import com.game.business.service.IAppUserService;
import com.game.business.vo.AppAgentTeamVo;
import com.game.common.core.domain.HttpRet;
import com.game.common.utils.DateUtils;
import com.game.common.utils.MessageUtils;
import com.game.common.utils.SecurityUtils;
import com.game.common.utils.StringUtils;
import com.github.pagehelper.PageHelper;
import eu.bitwalker.useragentutils.UserAgent;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.game.business.mapper.AppUserAgentMapper;
import com.game.business.domain.AppUserAgent;
import com.game.business.service.IAppUserAgentService;
import com.game.common.annotation.DataSource;
import com.game.common.enums.DataSourceType;

/**
 * 用户代理表Service业务层处理
 *
 * @author game
 * @date 2024-06-25
 */
@Service
public class AppUserAgentServiceImpl extends ServiceImpl<AppUserAgentMapper, AppUserAgent> implements IAppUserAgentService {

    @Autowired
    private AppUserAgentMapper appUserAgentMapper;

    @Autowired
    private IAppUserService appUserService;

    @Autowired
    private IAppGameCommissionService gameCommissionService;

    @Autowired
    private IAppGameService appGameService;

    /**
     * 查询用户代理表
     *
     * @param id 用户代理表主键
     * @return 用户代理表
     */
    @Override
    public AppUserAgent selectAppUserAgentById(Long id) {
        return appUserAgentMapper.selectAppUserAgentById(id);
    }

    /**
     * 查询用户代理表列表
     *
     * @param appUserAgent 用户代理表
     * @return 用户代理表
     */
    @Override
    public List<AppUserAgent> selectAppUserAgentList(AppUserAgent appUserAgent) {
        return appUserAgentMapper.selectAppUserAgentList(appUserAgent);
    }

    /**
     * 新增用户代理表
     *
     * @param appUserAgent 用户代理表
     * @return 结果
     */
    @Override
    public int insertAppUserAgent(AppUserAgent appUserAgent) {
                appUserAgent.setCreateTime(DateUtils.getNowDate());
            return appUserAgentMapper.insertAppUserAgent(appUserAgent);
    }

    /**
     * 修改用户代理表
     *
     * @param appUserAgent 用户代理表
     * @return 结果
     */
    @Override
    public int updateAppUserAgent(AppUserAgent appUserAgent) {
                appUserAgent.setUpdateTime(DateUtils.getNowDate());
        return appUserAgentMapper.updateAppUserAgent(appUserAgent);
    }

    /**
     * 批量删除用户代理表
     *
     * @param ids 需要删除的用户代理表主键
     * @return 结果
     */
    @Override
    public int deleteAppUserAgentByIds(Long[] ids) {
        return appUserAgentMapper.deleteAppUserAgentByIds(ids);
    }

    /**
     * 删除用户代理表信息
     *
     * @param id 用户代理表主键
     * @return 结果
     */
    @Override
    public int deleteAppUserAgentById(Long id) {
        return appUserAgentMapper.deleteAppUserAgentById(id);
    }

    /**
     * 统计用户代理人数
     * */
    @Override
    public int countAgent(Long userId, String beginTime, String endTime) {
        LambdaQueryWrapper<AppUserAgent> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(AppUserAgent::getPid,userId);
        if(StringUtils.isNotBlank(beginTime)){
            queryWrapper.ge(AppUserAgent::getCreateTime,beginTime);
        }
        if(StringUtils.isNotBlank(endTime)){
            queryWrapper.le(AppUserAgent::getCreateTime,endTime);
        }
        queryWrapper.eq(AppUserAgent::getAuditStatus,"1");
        return appUserAgentMapper.selectCount(queryWrapper).intValue();
    }

    /**
     * 统计用户团队投注金额
     * */
    @Override
    public double sumBetting(String beginTime,String endTime,Long pid) {
        return appUserAgentMapper.sumBetting(beginTime,endTime,pid);
    }

    @Override
    public HttpRet<Boolean> updateSetting(AppAgentEditDto editDto) {
        Long userId = SecurityUtils.getUserId();
        if(null != editDto.getUserId()){
            userId = editDto.getUserId();
        }
        LambdaQueryWrapper<AppUserAgent>  queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(AppUserAgent::getUserId,userId);
        AppUserAgent userAgent = appUserAgentMapper.selectOne(queryWrapper);
        if(userAgent == null){
            return HttpRet.fail("当前用户不是代理无法进行设置");
        }
        if(null != editDto.getUserId() && !SecurityUtils.getUserId().equals(userAgent.getPid())){
            return HttpRet.fail("只能设置自己的下级代理");
        }
        if(!userAgent.getPid().equals(0L)){
            //当前用户有上级
            //查询上级代理设置
            LambdaQueryWrapper<AppUserAgent>  queryWrapper2 = new LambdaQueryWrapper<>();
            queryWrapper2.eq(AppUserAgent::getUserId,userId);
            AppUserAgent pUser = appUserAgentMapper.selectOne(queryWrapper2);
            if(null != editDto.getLiveRate() && editDto.getLiveRate().longValue() >= pUser.getLiveRate().longValue()){
                return HttpRet.fail("直播分成比例不能大于等于".concat(String.valueOf(pUser.getLiveRate())));
            }
            if(null != editDto.getDividendGuaranteeRate() && editDto.getDividendGuaranteeRate().longValue() >= pUser.getDividendGuaranteeRate().longValue()){
                return HttpRet.fail("保底分成比例不能大于等于".concat(String.valueOf(pUser.getLiveRate())));
            }
        }
        AppUserAgent update = new AppUserAgent();
        BeanUtils.copyProperties(editDto,update);
        update.setId(userAgent.getId());
        appUserAgentMapper.updateById(update);
        return HttpRet.success(MessageUtils.message("11022"),true);
    }

    @Override
    public List<AppAgentTeamVo> teamList(AppAgentTeamDto teamDto) {
        teamDto.setPid(SecurityUtils.getUserId());
        List<AppAgentTeamVo> list = appUserAgentMapper.teamList(teamDto);
        if(null != list && list.size() > 0){
            List<Long> ids = list.stream().map(e->{return e.getUserId();}).collect(Collectors.toList());
            //查询游戏分红比例
            List<AppGameCommission> gameCommissionList = gameCommissionService.getByIds(ids);
            List<AppGame> gameList = appGameService.selectAppGameList(new AppGame());
            if(null != gameCommissionList && gameCommissionList.size() > 0){
                Map<String,List<AppGameCommission>> map = gameCommissionList.stream().
                        collect(Collectors.groupingBy(j->String.valueOf(j.getUserId())));
                list.forEach(e->{
                    if(null != map.get(String.valueOf(e.getUserId()))){
                        e.setGameCommissionList(map.get(String.valueOf(e.getUserId())));
                    }else if(null != gameList && gameList.size() > 0){
                        e.setGameCommissionList(gameList.stream().map(j->{
                            AppGameCommission gameCommission = new AppGameCommission();
                            gameCommission.setUserId(e.getUserId());
                            gameCommission.setGameId(j.getId());
                            gameCommission.setGameName(j.getName());
                            gameCommission.setPid(e.getPid());
                            gameCommission.setGameRate(0L);
                            return gameCommission;
                        }).collect(Collectors.toList()));
                    }
                });
            }
        }
        return list;
    }

}